import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/common/dialog_dropdown.dart';
import 'package:syncrow_web/common/dialog_textfield_dropdown.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/dialog/create_space_model_dialog.dart';
import 'package:syncrow_web/pages/spaces_management/tag_model/views/add_device_type_model_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class AssignTagModelsDialog extends StatelessWidget {
  final List<ProductModel>? products;
  final List<SubspaceTemplateModel>? subspaces;
  final SpaceTemplateModel? spaceModel;

  final List<TagModel> initialTags;
  final ValueChanged<List<TagModel>>? onTagsAssigned;
  final List<SelectedProduct> addedProducts;
  final List<String>? allTags;
  final String spaceName;
  final String title;
  final BuildContext? pageContext;
  final List<String>? otherSpaceModels;

  const AssignTagModelsDialog(
      {Key? key,
      required this.products,
      required this.subspaces,
      required this.addedProducts,
      required this.initialTags,
      this.onTagsAssigned,
      this.allTags,
      required this.spaceName,
      required this.title,
      this.pageContext,
      this.otherSpaceModels,
      this.spaceModel})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<String> locations = (subspaces ?? [])
        .map((subspace) => subspace.subspaceName)
        .toList()
      ..add('Main Space');

    return BlocProvider(
        create: (_) => AssignTagModelBloc()
          ..add(InitializeTagModels(
            initialTags: initialTags,
            addedProducts: addedProducts,
          )),
        child: BlocListener<AssignTagModelBloc, AssignTagModelState>(
          listener: (context, state) {},
          child: BlocBuilder<AssignTagModelBloc, AssignTagModelState>(
            builder: (context, state) {
              if (state is AssignTagModelLoaded) {
                final controllers = List.generate(
                  state.tags.length,
                  (index) => TextEditingController(text: state.tags[index].tag),
                );

                return AlertDialog(
                  title: Text(title),
                  backgroundColor: ColorsManager.whiteColors,
                  content: SingleChildScrollView(
                    child: Column(
                      children: [
                        ClipRRect(
                          borderRadius: BorderRadius.circular(20),
                          child: DataTable(
                            headingRowColor: WidgetStateProperty.all(
                                ColorsManager.dataHeaderGrey),
                            border: TableBorder.all(
                              color: ColorsManager.dataHeaderGrey,
                              width: 1,
                              borderRadius: BorderRadius.circular(20),
                            ),
                            columns: [
                              DataColumn(
                                  label: Text('#',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  label: Text('Device',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  numeric: false,
                                  label: Text('Tag',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                              DataColumn(
                                  label: Text('Location',
                                      style: Theme.of(context)
                                          .textTheme
                                          .bodyMedium)),
                            ],
                            rows: state.tags.isEmpty
                                ? [
                                    const DataRow(cells: [
                                      DataCell(
                                        Center(
                                          child: Text(
                                            'No Data Available',
                                            style: TextStyle(
                                              fontSize: 14,
                                              color:
                                                  ColorsManager.lightGrayColor,
                                            ),
                                          ),
                                        ),
                                      ),
                                      DataCell(SizedBox()),
                                      DataCell(SizedBox()),
                                      DataCell(SizedBox()),
                                    ])
                                  ]
                                : List.generate(state.tags.length, (index) {
                                    final tag = state.tags[index];
                                    final controller = controllers[index];
                                    final availableTags = getAvailableTags(
                                        allTags ?? [], state.tags, tag);

                                    return DataRow(
                                      cells: [
                                        DataCell(Text((index + 1).toString())),
                                        DataCell(
                                          Row(
                                            mainAxisAlignment:
                                                MainAxisAlignment.spaceBetween,
                                            children: [
                                              Expanded(
                                                  child: Text(
                                                tag.product?.name ?? 'Unknown',
                                                overflow: TextOverflow.ellipsis,
                                              )),
                                              const SizedBox(width: 10),
                                              Container(
                                                width: 20.0,
                                                height: 20.0,
                                                decoration: BoxDecoration(
                                                  shape: BoxShape.circle,
                                                  border: Border.all(
                                                    color: ColorsManager
                                                        .lightGrayColor,
                                                    width: 1.0,
                                                  ),
                                                ),
                                                child: IconButton(
                                                  icon: const Icon(
                                                    Icons.close,
                                                    color: ColorsManager
                                                        .lightGreyColor,
                                                    size: 16,
                                                  ),
                                                  onPressed: () {
                                                    context
                                                        .read<
                                                            AssignTagModelBloc>()
                                                        .add(DeleteTagModel(
                                                            tagToDelete: tag,
                                                            tags: state.tags));
                                                  },
                                                  tooltip: 'Delete Tag',
                                                  padding: EdgeInsets.zero,
                                                  constraints:
                                                      const BoxConstraints(),
                                                ),
                                              ),
                                            ],
                                          ),
                                        ),
                                        DataCell(
                                          Container(
                                            alignment: Alignment
                                                .centerLeft, // Align cell content to the left
                                            child: SizedBox(
                                              width: double
                                                  .infinity, // Ensure full width for dropdown
                                              child: DialogTextfieldDropdown(
                                                items: availableTags,
                                                initialValue: tag.tag,
                                                onSelected: (value) {
                                                  controller.text = value;
                                                  context
                                                      .read<
                                                          AssignTagModelBloc>()
                                                      .add(UpdateTag(
                                                        index: index,
                                                        tag: value,
                                                      ));
                                                },
                                              ),
                                            ),
                                          ),
                                        ),
                                        DataCell(
                                          SizedBox(
                                              width: double.infinity,
                                              child: DialogDropdown(
                                                items: locations,
                                                selectedValue:
                                                    tag.location ?? 'Main Space',
                                                onSelected: (value) {
                                                  context
                                                      .read<
                                                          AssignTagModelBloc>()
                                                      .add(UpdateLocation(
                                                        index: index,
                                                        location: value,
                                                      ));
                                                },
                                              )),
                                        ),
                                      ],
                                    );
                                  }),
                          ),
                        ),
                        if (state.errorMessage != null)
                          Text(
                            state.errorMessage!,
                            style: const TextStyle(
                                color: ColorsManager.warningRed),
                          ),
                      ],
                    ),
                  ),
                  actions: [
                    Row(
                      mainAxisAlignment: MainAxisAlignment.spaceAround,
                      children: [
                        const SizedBox(width: 10),
                        Expanded(
                          child: Builder(
                            builder: (buttonContext) => CancelButton(
                              label: 'Add New Device',
                              onPressed: () async {
                                for (var tag in state.tags) {
                                  if (tag.location == null ||
                                      subspaces == null) {
                                    continue;
                                  }

                                  final previousTagSubspace =
                                      checkTagExistInSubspace(
                                          tag, subspaces ?? []);

                                  if (tag.location == 'Main Space') {
                                    removeTagFromSubspace(
                                        tag, previousTagSubspace);
                                  } else if (tag.location !=
                                      previousTagSubspace?.subspaceName) {
                                    removeTagFromSubspace(
                                        tag, previousTagSubspace);
                                    moveToNewSubspace(tag, subspaces ?? []);
                                    state.tags.removeWhere(
                                        (t) => t.internalId == tag.internalId);
                                  } else {
                                    updateTagInSubspace(
                                        tag, previousTagSubspace);
                                    state.tags.removeWhere(
                                        (t) => t.internalId == tag.internalId);
                                  }
                                }
                                if (context.mounted) {
                                  Navigator.of(context).pop();

                                  await showDialog<bool>(
                                    barrierDismissible: false,
                                    context: context,
                                    builder: (dialogContext) =>
                                        AddDeviceTypeModelWidget(
                                            products: products,
                                            subspaces: subspaces,
                                            isCreate: false,
                                            initialSelectedProducts:
                                                addedProducts,
                                            allTags: allTags,
                                            spaceName: spaceName,
                                            otherSpaceModels: otherSpaceModels,
                                            spaceTagModels: state.tags,
                                            pageContext: pageContext,
                                            spaceModel: SpaceTemplateModel(
                                                modelName: spaceName,
                                                tags: state.tags,
                                                uuid: spaceModel?.uuid,
                                                internalId:
                                                    spaceModel?.internalId,
                                                subspaceModels: subspaces)),
                                  );
                                }
                              },
                            ),
                          ),
                        ),
                        const SizedBox(width: 10),
                        Expanded(
                          child: DefaultButton(
                            borderRadius: 10,
                            backgroundColor: state.isSaveEnabled
                                ? ColorsManager.secondaryColor
                                : ColorsManager.grayColor,
                            foregroundColor: ColorsManager.whiteColors,
                            onPressed: state.isSaveEnabled
                                ? () async {
                                    for (var tag in state.tags) {
                                      if (tag.location == null ||
                                          subspaces == null) {
                                        continue;
                                      }

                                      final previousTagSubspace =
                                          checkTagExistInSubspace(
                                              tag, subspaces ?? []);

                                      if (tag.location == 'Main Space') {
                                        removeTagFromSubspace(
                                            tag, previousTagSubspace);
                                      } else if (tag.location !=
                                          previousTagSubspace?.subspaceName) {
                                        removeTagFromSubspace(
                                            tag, previousTagSubspace);
                                        moveToNewSubspace(tag, subspaces ?? []);
                                        state.tags.removeWhere((t) =>
                                            t.internalId == tag.internalId);
                                      } else {
                                        updateTagInSubspace(
                                            tag, previousTagSubspace);
                                        state.tags.removeWhere((t) =>
                                            t.internalId == tag.internalId);
                                      }
                                    }
                                    Navigator.of(context)
                                        .popUntil((route) => route.isFirst);

                                    await showDialog(
                                      context: context,
                                      builder: (BuildContext dialogContext) {
                                        return CreateSpaceModelDialog(
                                          products: products,
                                          allTags: allTags,
                                          pageContext: pageContext,
                                          otherSpaceModels: otherSpaceModels,
                                          spaceModel: SpaceTemplateModel(
                                              modelName: spaceName,
                                              tags: state.tags,
                                              uuid: spaceModel?.uuid,
                                              internalId:
                                                  spaceModel?.internalId,
                                              subspaceModels: subspaces),
                                        );
                                      },
                                    );
                                  }
                                : null,
                            child: const Text('Save'),
                          ),
                        ),
                        const SizedBox(width: 10),
                      ],
                    ),
                  ],
                );
              } else if (state is AssignTagModelLoading) {
                return const Center(child: CircularProgressIndicator());
              } else {
                return const Center(child: Text('Something went wrong.'));
              }
            },
          ),
        ));
  }

  List<String> getAvailableTags(
      List<String> allTags, List<TagModel> currentTags, TagModel currentTag) {
    return allTags
        .where((tagValue) => !currentTags
            .where((e) => e != currentTag) // Exclude the current row
            .map((e) => e.tag)
            .contains(tagValue))
        .toList();
  }

  void removeTagFromSubspace(TagModel tag, SubspaceTemplateModel? subspace) {
    subspace?.tags?.removeWhere((t) => t.internalId == tag.internalId);
  }

  SubspaceTemplateModel? checkTagExistInSubspace(
      TagModel tag, List<SubspaceTemplateModel>? subspaces) {
    if (subspaces == null) return null;
    for (var subspace in subspaces) {
      if (subspace.tags == null) return null;
      for (var t in subspace.tags!) {
        if (tag.internalId == t.internalId) return subspace;
      }
    }
    return null;
  }

  void moveToNewSubspace(TagModel tag, List<SubspaceTemplateModel> subspaces) {
    final targetSubspace = subspaces
        .firstWhere((subspace) => subspace.subspaceName == tag.location);

    targetSubspace.tags ??= [];
    if (targetSubspace.tags?.any((t) => t.internalId == tag.internalId) !=
        true) {
      targetSubspace.tags?.add(tag);
    }
  }

  void updateTagInSubspace(TagModel tag, SubspaceTemplateModel? subspace) {
    final currentTag = subspace?.tags?.firstWhere(
      (t) => t.internalId == tag.internalId,
    );
    if (currentTag != null) {
      currentTag.tag = tag.tag;
    }
  }
}
